local frame = CreateFrame("Frame")

frame:RegisterEvent("PLAYER_ENTERING_WORLD") -- Fired when the player logs in, /reloads the UI, or zones between map instances, basically whenever the loading screen appears.
frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED") -- Fired when a spell is cast successfully. Event is received even if spell is resisted.
frame:RegisterEvent("ARENA_TEAM_ROSTER_UPDATE") -- Requests the arena team information from the server
frame:RegisterEvent("GROUP_ROSTER_UPDATE") -- Fired whenever a group or raid is formed or disbanded, players are leaving or joining the group or raid.

frame:SetScript("OnEvent", function(self, event, arg1, arg2, arg3)
	-- When Greater Rune of Warding is applied, save the timestamp 60 mins from now
	if event == "UNIT_SPELLCAST_SUCCEEDED" and arg3 == 32282 then
		RuneTimestamp = time() + 3600
	end
	
	-- When the party reloads or changes zone
	if event == "PLAYER_ENTERING_WORLD" then
		if checkArenaParty() then
			printRuneTimer()
		end
	end
	
	-- When the party composition changes, check again and print if needed
	if event == "GROUP_ROSTER_UPDATE" then
		if checkArenaParty() then
			printRuneTimer()
		end
	end
	
	-- Update the arena team information stored SavedVariables for all the player's teams
	if event == "ARENA_TEAM_ROSTER_UPDATE" then
		local teamsArray = {}
		for teamId = 1, 3, 1 do
			local maxPlayers = 0
			if teamId == 1 then
				teamsArray[1] = {}
				maxPlayers = 4
			elseif teamId == 2 then
				teamsArray[2] = {}
				maxPlayers = 6
			elseif teamId == 3 then
				teamsArray[3] = {}
				maxPlayers = 10
			end
			
			for playerId = 1, maxPlayers, 1 do
				name, rank, level, class, online, played, win, seasonPlayed, seasonWin, personalRating = GetArenaTeamRosterInfo(teamId, playerId);
				if name then
					teamsArray[teamId][playerId] = name
				end
			end
		end
		-- Save to SavedVariables
		ArenaTeam = teamsArray
	end
	
end)

-- Print the remaining time on Greater Rune of Warding
function printRuneTimer()
	local spellId = 32282 -- https://tbc.wowhead.com/spell=32282/greater-rune-of-warding
	local _, _, icon = GetSpellInfo(spellId)
	local RuneTimeleft = (RuneTimestamp - time()) / 60
	if RuneTimeleft > 0 then
		print("|T"..icon..":0|t " .. truncate(RuneTimeleft, 0) .. " minutes left on " .. GetSpellLink(spellId) .. "!")
	else
		print("|T"..icon..":0|t " .. GetSpellLink(spellId) .. " missing!")
	end
end

-- Check if current party is a party with same members as present in the arena team of the same size
function checkArenaParty()
	local playingArena = false
	
	-- Check for 2v2
	if GetNumGroupMembers() == 2 then
		for i = 1, GetNumGroupMembers(), 1 do
			if has_value(ArenaTeam[1], UnitName("party"..i)) then
				playingArena = true
			end
		end
	-- Check for 3v3
	elseif GetNumGroupMembers() == 3 then
		for i = 1, GetNumGroupMembers(), 1 do
			if has_value(ArenaTeam[2], UnitName("party"..i)) then
				playingArena = true
			end
		end
	-- Check for 5v5
	elseif GetNumGroupMembers() == 5 then
		for i = 1, GetNumGroupMembers(), 1 do
			if has_value(ArenaTeam[3], UnitName("party"..i)) then
				playingArena = true
			end
		end
	end
	
	return playingArena -- If all members in the current party are in the arena team of the same size as the party this returns true, otherwise false is returned
end

function truncate(number, decimals)
    return number - (number % (0.1 ^ decimals))
end

function has_value (tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end

    return false
end